@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2015, 2016, 2017, 2018 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
 #include "applications-page.h"
 
 #include "category.h"
+#include "category-button.h"
 #include "launcher.h"
 #include "launcher-view.h"
-#include "section-button.h"
 #include "settings.h"
 #include "slot.h"
 #include "window.h"
@@ -36,40 +36,20 @@ using namespace WhiskerMenu;
 
 //-----------------------------------------------------------------------------
 
-enum
-{
-	STATUS_INVALID,
-	STATUS_LOADING,
-	STATUS_LOADING_RELOAD,
-	STATUS_LOADED
-};
-
-//-----------------------------------------------------------------------------
-
 ApplicationsPage::ApplicationsPage(Window* window) :
 	Page(window),
-	m_garcon_menu(NULL),
-	m_garcon_settings_menu(NULL),
-	m_load_status(STATUS_INVALID)
+	m_garcon_menu(nullptr),
+	m_garcon_settings_menu(nullptr),
+	m_status(LoadStatus::Invalid)
 {
-	// Set desktop environment for applications
-	const gchar* desktop = g_getenv("XDG_CURRENT_DESKTOP");
-	if (G_LIKELY(!desktop))
-	{
-		desktop = "XFCE";
-	}
-	else if (*desktop == '\0')
-	{
-		desktop = NULL;
-	}
-	garcon_set_environment(desktop);
+	garcon_set_environment_xdg(GARCON_ENVIRONMENT_XFCE);
 }
 
 //-----------------------------------------------------------------------------
 
 ApplicationsPage::~ApplicationsPage()
 {
-	clear_applications();
+	clear();
 }
 
 //-----------------------------------------------------------------------------
@@ -79,24 +59,24 @@ GtkTreeModel* ApplicationsPage::create_launcher_model(std::vector<std::string>&
 	// Create new model for treeview
 	GtkListStore* store = gtk_list_store_new(
 			LauncherView::N_COLUMNS,
-			G_TYPE_STRING,
+			G_TYPE_ICON,
 			G_TYPE_STRING,
 			G_TYPE_STRING,
 			G_TYPE_POINTER);
 
 	// Fetch menu items or remove them from list if missing
-	for (std::vector<std::string>::iterator i = desktop_ids.begin(); i != desktop_ids.end(); ++i)
+	for (auto i = desktop_ids.begin(); i != desktop_ids.end(); ++i)
 	{
 		if (i->empty())
 		{
 			continue;
 		}
 
-		Launcher* launcher = get_application(*i);
+		Launcher* launcher = find(*i);
 		if (launcher)
 		{
 			gtk_list_store_insert_with_values(
-					store, NULL, G_MAXINT,
+					store, nullptr, G_MAXINT,
 					LauncherView::COLUMN_ICON, launcher->get_icon(),
 					LauncherView::COLUMN_TEXT, launcher->get_text(),
 					LauncherView::COLUMN_TOOLTIP, launcher->get_tooltip(),
@@ -116,10 +96,24 @@ GtkTreeModel* ApplicationsPage::create_launcher_model(std::vector<std::string>&
 
 //-----------------------------------------------------------------------------
 
-Launcher* ApplicationsPage::get_application(const std::string& desktop_id) const
+Launcher* ApplicationsPage::find(const std::string& desktop_id) const
 {
-	std::map<std::string, Launcher*>::const_iterator i = m_items.find(desktop_id);
-	return (i != m_items.end()) ? i->second : NULL;
+	auto i = m_items.find(desktop_id);
+	return (i != m_items.end()) ? i->second : nullptr;
+}
+
+//-----------------------------------------------------------------------------
+
+std::vector<Launcher*> ApplicationsPage::find_all() const
+{
+	std::vector<Launcher*> launchers;
+	launchers.reserve(m_items.size());
+	for (const auto& i : m_items)
+	{
+		launchers.push_back(i.second);
+	}
+	std::sort(launchers.begin(), launchers.end(), &Element::less_than);
+	return launchers;
 }
 
 //-----------------------------------------------------------------------------
@@ -127,18 +121,18 @@ Launcher* ApplicationsPage::get_application(const std::string& desktop_id) const
 void ApplicationsPage::apply_filter(GtkToggleButton* togglebutton)
 {
 	// Only apply filter for active button
-	if (gtk_toggle_button_get_active(togglebutton) == false)
+	if (!gtk_toggle_button_get_active(togglebutton))
 	{
 		return;
 	}
 
 	// Find category matching button
-	Category* category = NULL;
-	for (std::vector<Category*>::const_iterator i = m_categories.begin(), end = m_categories.end(); i != end; ++i)
+	Category* category = nullptr;
+	for (auto test_category : m_categories)
 	{
-		if (GTK_TOGGLE_BUTTON((*i)->get_button()->get_button()) == togglebutton)
+		if (GTK_TOGGLE_BUTTON(test_category->get_button()->get_widget()) == togglebutton)
 		{
-			category = *i;
+			category = test_category;
 			break;
 		}
 	}
@@ -155,40 +149,40 @@ void ApplicationsPage::apply_filter(GtkToggleButton* togglebutton)
 
 //-----------------------------------------------------------------------------
 
-void ApplicationsPage::invalidate_applications()
+void ApplicationsPage::invalidate()
 {
-	if (m_load_status == STATUS_LOADED)
+	if (m_status == LoadStatus::Done)
 	{
-		m_load_status = STATUS_INVALID;
+		m_status = LoadStatus::Invalid;
 	}
-	else if (m_load_status == STATUS_LOADING)
+	else if (m_status == LoadStatus::Loading)
 	{
-		m_load_status = STATUS_LOADING_RELOAD;
+		m_status = LoadStatus::ReloadRequired;
 	}
 }
 
 //-----------------------------------------------------------------------------
 
-bool ApplicationsPage::load_applications()
+bool ApplicationsPage::load()
 {
 	// Check if already loaded
-	if (m_load_status == STATUS_LOADED)
+	if (m_status == LoadStatus::Done)
 	{
 		return true;
 	}
 	// Check if currently loading
-	else if ((m_load_status == STATUS_LOADING) || (m_load_status == STATUS_LOADING_RELOAD))
+	else if (m_status != LoadStatus::Invalid)
 	{
 		return false;
 	}
-	m_load_status = STATUS_LOADING;
+	m_status = LoadStatus::Loading;
 
 	// Load menu
-	clear_applications();
+	clear();
 
 	// Load contents in thread if possible
-	GTask* task = g_task_new(NULL, NULL, &ApplicationsPage::load_contents_slot, this);
-	g_task_set_task_data(task, this, NULL);
+	GTask* task = g_task_new(nullptr, nullptr, &ApplicationsPage::load_contents_slot, this);
+	g_task_set_task_data(task, this, nullptr);
 	g_task_run_in_thread(task, &ApplicationsPage::load_garcon_menu_slot);
 	g_object_unref(task);
 
@@ -199,20 +193,20 @@ bool ApplicationsPage::load_applications()
 
 void ApplicationsPage::reload_category_icon_size()
 {
-	for (std::vector<Category*>::const_iterator i = m_categories.begin(), end = m_categories.end(); i != end; ++i)
+	for (auto category : m_categories)
 	{
-		(*i)->get_button()->reload_icon_size();
+		category->get_button()->reload_icon_size();
 	}
 }
 
 //-----------------------------------------------------------------------------
 
-void ApplicationsPage::clear_applications()
+void ApplicationsPage::clear()
 {
 	// Free categories
-	for (std::vector<Category*>::iterator i = m_categories.begin(), end = m_categories.end(); i != end; ++i)
+	for (auto category : m_categories)
 	{
-		delete *i;
+		delete category;
 	}
 	m_categories.clear();
 
@@ -220,9 +214,9 @@ void ApplicationsPage::clear_applications()
 	get_window()->unset_items();
 	get_view()->unset_model();
 
-	for (std::map<std::string, Launcher*>::iterator i = m_items.begin(), end = m_items.end(); i != end; ++i)
+	for (const auto& i : m_items)
 	{
-		delete i->second;
+		delete i.second;
 	}
 	m_items.clear();
 
@@ -230,14 +224,14 @@ void ApplicationsPage::clear_applications()
 	if (G_LIKELY(m_garcon_menu))
 	{
 		g_object_unref(m_garcon_menu);
-		m_garcon_menu = NULL;
+		m_garcon_menu = nullptr;
 	}
 
 	// Free settings menu
 	if (G_LIKELY(m_garcon_settings_menu))
 	{
 		g_object_unref(m_garcon_settings_menu);
-		m_garcon_settings_menu = NULL;
+		m_garcon_settings_menu = nullptr;
 	}
 }
 
@@ -256,10 +250,10 @@ void ApplicationsPage::load_garcon_menu()
 	}
 
 	// Load menu
-	if (m_garcon_menu && !garcon_menu_load(m_garcon_menu, NULL, NULL))
+	if (m_garcon_menu && !garcon_menu_load(m_garcon_menu, nullptr, nullptr))
 	{
 		g_object_unref(m_garcon_menu);
-		m_garcon_menu = NULL;
+		m_garcon_menu = nullptr;
 	}
 
 	if (!m_garcon_menu)
@@ -267,42 +261,40 @@ void ApplicationsPage::load_garcon_menu()
 		return;
 	}
 
-	g_signal_connect_slot<GarconMenu*>(m_garcon_menu, "reload-required", &ApplicationsPage::invalidate_applications, this);
-	load_menu(m_garcon_menu, NULL);
+	g_signal_connect_slot<GarconMenu*>(m_garcon_menu, "reload-required", &ApplicationsPage::invalidate, this);
+	load_menu(m_garcon_menu, nullptr, wm_settings->load_hierarchy);
 
 	// Create settings menu
 	gchar* path = xfce_resource_lookup(XFCE_RESOURCE_CONFIG, "menus/xfce-settings-manager.menu");
-	m_garcon_settings_menu = garcon_menu_new_for_path(path != NULL ? path : SETTINGS_MENUFILE);
+	m_garcon_settings_menu = garcon_menu_new_for_path(path ? path : SETTINGS_MENUFILE);
 	g_free(path);
 
 	if (m_garcon_settings_menu)
 	{
-		g_signal_connect_slot<GarconMenu*>(m_garcon_settings_menu, "reload-required", &ApplicationsPage::invalidate_applications, this);
+		g_signal_connect_slot<GarconMenu*>(m_garcon_settings_menu, "reload-required", &ApplicationsPage::invalidate, this);
 	}
 
 	// Load settings menu
-	if (m_garcon_settings_menu && garcon_menu_load(m_garcon_settings_menu, NULL, NULL))
+	if (m_garcon_settings_menu && garcon_menu_load(m_garcon_settings_menu, nullptr, nullptr))
 	{
-		load_menu(m_garcon_settings_menu, NULL);
+		Category* category = new Category(nullptr);
+		load_menu(m_garcon_settings_menu, category, false);
+		delete category;
 	}
 
 	// Sort items and categories
 	if (!wm_settings->load_hierarchy)
 	{
-		for (std::vector<Category*>::const_iterator i = m_categories.begin(), end = m_categories.end(); i != end; ++i)
+		for (auto category : m_categories)
 		{
-			(*i)->sort();
+			category->sort();
 		}
 		std::sort(m_categories.begin(), m_categories.end(), &Element::less_than);
 	}
 
 	// Create all items category
-	Category* category = new Category(NULL);
-	for (std::map<std::string, Launcher*>::const_iterator i = m_items.begin(), end = m_items.end(); i != end; ++i)
-	{
-		category->append_item(i->second);
-	}
-	category->sort();
+	Category* category = new Category(nullptr);
+	category->append_items(find_all());
 	m_categories.insert(m_categories.begin(), category);
 }
 
@@ -314,7 +306,7 @@ void ApplicationsPage::load_contents()
 	{
 		get_window()->set_loaded();
 
-		m_load_status = STATUS_INVALID;
+		m_status = LoadStatus::Invalid;
 
 		return;
 	}
@@ -324,11 +316,11 @@ void ApplicationsPage::load_contents()
 	get_view()->set_model(m_categories.front()->get_model());
 
 	// Add buttons for categories
-	std::vector<SectionButton*> category_buttons;
-	for (std::vector<Category*>::const_iterator i = m_categories.begin(), end = m_categories.end(); i != end; ++i)
+	std::vector<CategoryButton*> category_buttons;
+	for (auto category : m_categories)
 	{
-		SectionButton* category_button = (*i)->get_button();
-		g_signal_connect_slot(category_button->get_button(), "toggled", &ApplicationsPage::apply_filter, this);
+		CategoryButton* category_button = category->get_button();
+		g_signal_connect_slot(category_button->get_widget(), "toggled", &ApplicationsPage::apply_filter, this);
 		category_buttons.push_back(category_button);
 	}
 
@@ -339,98 +331,101 @@ void ApplicationsPage::load_contents()
 	get_window()->set_items();
 	get_window()->set_loaded();
 
-	m_load_status = (m_load_status == STATUS_LOADING) ? STATUS_LOADED : STATUS_INVALID;
+	m_status = (m_status == LoadStatus::Loading) ? LoadStatus::Done : LoadStatus::Invalid;
 }
 
 //-----------------------------------------------------------------------------
 
-void ApplicationsPage::load_menu(GarconMenu* menu, Category* parent_category)
+bool ApplicationsPage::load_menu(GarconMenu* menu, Category* parent_category, bool load_hierarchy)
 {
-	GarconMenuDirectory* directory = garcon_menu_get_directory(menu);
-
-	// Skip hidden categories
-	if (directory && !garcon_menu_directory_get_visible(directory))
-	{
-		return;
-	}
-
-	// Track categories
-	bool first_level = directory && (garcon_menu_get_parent(menu) == m_garcon_menu);
-	Category* category = NULL;
-	if (directory)
-	{
-		if (first_level)
-		{
-			category = new Category(directory);
-			m_categories.push_back(category);
-		}
-		else if (!wm_settings->load_hierarchy)
-		{
-			category = parent_category;
-		}
-		else if (parent_category)
-		{
-			category = parent_category->append_menu(directory);
-		}
-	}
+	bool has_children = false;
 
 	// Add menu elements
 	GList* elements = garcon_menu_get_elements(menu);
-	for (GList* li = elements; li != NULL; li = li->next)
+	for (GList* li = elements; li; li = li->next)
 	{
+		// Add menu item
 		if (GARCON_IS_MENU_ITEM(li->data))
 		{
-			load_menu_item(GARCON_MENU_ITEM(li->data), category);
+			GarconMenuItem* menuitem = GARCON_MENU_ITEM(li->data);
+
+			// Listen for changes
+			g_signal_connect_slot<GarconMenuItem*>(menuitem, "changed", &ApplicationsPage::invalidate, this);
+
+			// Skip hidden items
+			if (!garcon_menu_element_get_visible(GARCON_MENU_ELEMENT(menuitem)))
+			{
+				continue;
+			}
+
+			// Create launcher
+			std::string desktop_id(garcon_menu_item_get_desktop_id(menuitem));
+			auto iter = m_items.find(desktop_id);
+			if (iter == m_items.end())
+			{
+				iter = m_items.emplace(std::move(desktop_id), new Launcher(menuitem)).first;
+			}
+
+			// Add launcher to current category
+			if (parent_category)
+			{
+				parent_category->append_item(iter->second);
+			}
+
+			has_children = true;
 		}
+		// Add separator
+		else if (GARCON_IS_MENU_SEPARATOR(li->data) && load_hierarchy && parent_category)
+		{
+			parent_category->append_separator();
+		}
+		// Add submenu
 		else if (GARCON_IS_MENU(li->data))
 		{
-			load_menu(GARCON_MENU(li->data), category);
-		}
-		else if (GARCON_IS_MENU_SEPARATOR(li->data) && wm_settings->load_hierarchy && category)
-		{
-			category->append_separator();
+			GarconMenu* submenu = GARCON_MENU(li->data);
+
+			// Skip hidden categories
+			GarconMenuDirectory* directory = garcon_menu_get_directory(submenu);
+			if (directory && !garcon_menu_directory_get_visible(directory))
+			{
+				continue;
+			}
+
+			// Create category
+			Category* category = nullptr;
+			if (!load_hierarchy && parent_category)
+			{
+				category = parent_category;
+			}
+			else
+			{
+				category = new Category(submenu);
+			}
+
+			// Populate category
+			if (load_menu(submenu, category, load_hierarchy))
+			{
+				if (!parent_category)
+				{
+					m_categories.push_back(category);
+				}
+				else if (category != parent_category)
+				{
+					parent_category->append_category(category);
+				}
+
+				has_children = true;
+			}
+			// Remove empty categories
+			else if (category != parent_category)
+			{
+				delete category;
+			}
 		}
 	}
 	g_list_free(elements);
 
-	// Free unused top-level categories
-	if (first_level && category->empty())
-	{
-		m_categories.erase(std::find(m_categories.begin(), m_categories.end(), category));
-		delete category;
-		category = NULL;
-	}
-
-	// Listen for menu changes
-	g_signal_connect_slot<GarconMenu*,GarconMenuDirectory*,GarconMenuDirectory*>(menu, "directory-changed", &ApplicationsPage::invalidate_applications, this);
-}
-
-//-----------------------------------------------------------------------------
-
-void ApplicationsPage::load_menu_item(GarconMenuItem* menu_item, Category* category)
-{
-	// Skip hidden items
-	if (!garcon_menu_element_get_visible(GARCON_MENU_ELEMENT(menu_item)))
-	{
-		return;
-	}
-
-	// Add to map
-	std::string desktop_id(garcon_menu_item_get_desktop_id(menu_item));
-	std::map<std::string, Launcher*>::iterator iter = m_items.find(desktop_id);
-	if (iter == m_items.end())
-	{
-		iter = m_items.insert(std::make_pair(desktop_id, new Launcher(menu_item))).first;
-	}
-
-	// Add menu item to current category
-	if (category)
-	{
-		category->append_item(iter->second);
-	}
-
-	// Listen for menu changes
-	g_signal_connect_slot<GarconMenuItem*>(menu_item, "changed", &ApplicationsPage::invalidate_applications, this);
+	return has_children;
 }
 
 //-----------------------------------------------------------------------------
