@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2015 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 
 #include "page.h"
 
-#include <map>
 #include <string>
+#include <unordered_map>
 #include <vector>
 
 #include <garcon/garcon.h>
@@ -30,7 +30,6 @@ namespace WhiskerMenu
 {
 
 class Category;
-class SectionButton;
 
 class ApplicationsPage : public Page
 {
@@ -40,26 +39,45 @@ public:
 	~ApplicationsPage();
 
 	GtkTreeModel* create_launcher_model(std::vector<std::string>& desktop_ids) const;
-	Launcher* get_application(const std::string& desktop_id) const;
+	Launcher* find(const std::string& desktop_id) const;
+	std::vector<Launcher*> find_all() const;
 
-	void invalidate_applications();
-	void load_applications();
+	void invalidate();
+	bool load();
 	void reload_category_icon_size();
 
 private:
 	void apply_filter(GtkToggleButton* togglebutton);
-	void clear_applications();
+	void clear();
 	void load_garcon_menu();
 	void load_contents();
-	void load_menu(GarconMenu* menu, Category* parent_category);
-	void load_menu_item(GarconMenuItem* menu_item, Category* category);
+	bool load_menu(GarconMenu* menu, Category* parent_category, bool load_hierarchy);
+
+	static void load_garcon_menu_slot(GTask* task, gpointer, gpointer task_data, GCancellable*)
+	{
+		static_cast<ApplicationsPage*>(task_data)->load_garcon_menu();
+		g_task_return_boolean(task, true);
+	}
+
+	static void load_contents_slot(GObject*, GAsyncResult*, gpointer user_data)
+	{
+		static_cast<ApplicationsPage*>(user_data)->load_contents();
+	}
 
 private:
 	GarconMenu* m_garcon_menu;
 	GarconMenu* m_garcon_settings_menu;
 	std::vector<Category*> m_categories;
-	std::map<std::string, Launcher*> m_items;
-	int m_load_status;
+	std::unordered_map<std::string, Launcher*> m_items;
+
+	enum class LoadStatus
+	{
+		Invalid,
+		Loading,
+		ReloadRequired,
+		Done
+	}
+	m_status;
 };
 
 }
