@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2016, 2018, 2020 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,6 +17,7 @@
 
 #include "command.h"
 
+#include "image-menu-item.h"
 #include "settings.h"
 #include "slot.h"
 
@@ -28,37 +29,26 @@ using namespace WhiskerMenu;
 
 //-----------------------------------------------------------------------------
 
-enum
-{
-	WHISKERMENU_COMMAND_UNCHECKED = -1,
-	WHISKERMENU_COMMAND_INVALID,
-	WHISKERMENU_COMMAND_VALID
-};
-
-//-----------------------------------------------------------------------------
-
 Command::Command(const gchar* icon, const gchar* text, const gchar* command, const gchar* error_text, const gchar* confirm_question, const gchar* confirm_status) :
-	m_button(NULL),
-	m_menuitem(NULL),
+	m_button(nullptr),
+	m_menuitem(nullptr),
 	m_icon(g_strdup(icon)),
 	m_mnemonic(g_strdup(text)),
 	m_command(g_strdup(command)),
 	m_error_text(g_strdup(error_text)),
-	m_status(WHISKERMENU_COMMAND_UNCHECKED),
 	m_shown(true),
-	m_timeout_details({NULL, g_strdup(confirm_question), g_strdup(confirm_status), 0})
+	m_status(CommandStatus::Unchecked),
+	m_timeout_details({nullptr, g_strdup(confirm_question), g_strdup(confirm_status), 0})
 {
-	std::string mnemonic(text ? text : "");
-	for (std::string::size_type i = 0, length = mnemonic.length(); i < length; ++i)
+	std::string tooltip(text ? text : "");
+	for (auto i = tooltip.begin(); i != tooltip.end(); ++i)
 	{
-		if (mnemonic[i] == '_')
+		if (*i == '_')
 		{
-			mnemonic.erase(i, 1);
-			--length;
-			--i;
+			i = tooltip.erase(i);
 		}
 	}
-	m_text = g_strdup(mnemonic.c_str());
+	m_text = g_strdup(tooltip.c_str());
 
 	check();
 }
@@ -105,7 +95,7 @@ GtkWidget* Command::get_button()
 	gtk_container_add(GTK_CONTAINER(m_button), GTK_WIDGET(image));
 
 	gtk_widget_set_visible(m_button, m_shown);
-	gtk_widget_set_sensitive(m_button, m_status == WHISKERMENU_COMMAND_VALID);
+	gtk_widget_set_sensitive(m_button, m_status == CommandStatus::Valid);
 
 	g_object_ref_sink(m_button);
 
@@ -121,15 +111,11 @@ GtkWidget* Command::get_menuitem()
 		return m_menuitem;
 	}
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-	m_menuitem = gtk_image_menu_item_new_with_mnemonic(m_mnemonic);
-	GtkWidget* image = gtk_image_new_from_icon_name(m_icon, GTK_ICON_SIZE_MENU);
-	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(m_menuitem), image);
+	m_menuitem = whiskermenu_image_menu_item_new_with_mnemonic(m_icon, m_mnemonic);
 	g_signal_connect_slot<GtkMenuItem*>(m_menuitem, "activate", &Command::activate, this);
-G_GNUC_END_IGNORE_DEPRECATIONS
 
 	gtk_widget_set_visible(m_menuitem, m_shown);
-	gtk_widget_set_sensitive(m_menuitem, m_status == WHISKERMENU_COMMAND_VALID);
+	gtk_widget_set_sensitive(m_menuitem, m_status == CommandStatus::Valid);
 
 	g_object_ref_sink(m_menuitem);
 
@@ -147,7 +133,7 @@ void Command::set(const gchar* command)
 
 	g_free(m_command);
 	m_command = g_strdup(command);
-	m_status = WHISKERMENU_COMMAND_UNCHECKED;
+	m_status = CommandStatus::Unchecked;
 	wm_settings->set_modified();
 }
 
@@ -177,31 +163,31 @@ void Command::set_shown(bool shown)
 
 void Command::check()
 {
-	if (m_status == WHISKERMENU_COMMAND_UNCHECKED)
+	if (m_status == CommandStatus::Unchecked)
 	{
 		gchar** argv;
-		if (g_shell_parse_argv(m_command, NULL, &argv, NULL))
+		if (g_shell_parse_argv(m_command, nullptr, &argv, nullptr))
 		{
 			gchar* path = g_find_program_in_path(argv[0]);
-			m_status = path ? WHISKERMENU_COMMAND_VALID : WHISKERMENU_COMMAND_INVALID;
+			m_status = path ? CommandStatus::Valid : CommandStatus::Invalid;
 			g_free(path);
 			g_strfreev(argv);
 		}
 		else
 		{
-			m_status = WHISKERMENU_COMMAND_INVALID;
+			m_status = CommandStatus::Invalid;
 		}
 	}
 
 	if (m_button)
 	{
 		gtk_widget_set_visible(m_button, m_shown);
-		gtk_widget_set_sensitive(m_button, m_status == WHISKERMENU_COMMAND_VALID);
+		gtk_widget_set_sensitive(m_button, m_status == CommandStatus::Valid);
 	}
 	if (m_menuitem)
 	{
 		gtk_widget_set_visible(m_menuitem, m_shown);
-		gtk_widget_set_sensitive(m_menuitem, m_status == WHISKERMENU_COMMAND_VALID);
+		gtk_widget_set_sensitive(m_menuitem, m_status == CommandStatus::Valid);
 	}
 }
 
@@ -217,10 +203,10 @@ void Command::activate()
 		return;
 	}
 
-	GError* error = NULL;
-	if (g_spawn_command_line_async(m_command, &error) == false)
+	GError* error = nullptr;
+	if (!g_spawn_command_line_async(m_command, &error))
 	{
-		xfce_dialog_show_error(NULL, error, m_error_text, NULL);
+		xfce_dialog_show_error(nullptr, error, m_error_text, nullptr);
 		g_error_free(error);
 	}
 }
@@ -231,26 +217,35 @@ void Command::activate()
 bool Command::confirm()
 {
 	// Create dialog
-	m_timeout_details.dialog = gtk_message_dialog_new(NULL, GtkDialogFlags(0),
+	m_timeout_details.dialog = gtk_message_dialog_new(nullptr, GtkDialogFlags(0),
 			GTK_MESSAGE_QUESTION, GTK_BUTTONS_CANCEL,
 			"%s", m_timeout_details.question);
 	GtkDialog* dialog = GTK_DIALOG(m_timeout_details.dialog);
 
 	GtkWindow* window = GTK_WINDOW(m_timeout_details.dialog);
+	gtk_window_set_icon_name(window, m_icon);
+	gtk_window_set_deletable(window, false);
 	gtk_window_set_keep_above(window, true);
-	gtk_window_stick(window);
 	gtk_window_set_skip_taskbar_hint(window, true);
-	gtk_window_set_title(window, m_text);
+	gtk_window_stick(window);
+
+	GtkWidget* header = gtk_header_bar_new();
+	gtk_header_bar_set_has_subtitle(GTK_HEADER_BAR(header), false);
+	gtk_header_bar_set_show_close_button(GTK_HEADER_BAR(header), false);
+	gtk_widget_show(header);
+	gtk_window_set_titlebar(window, header);
 
 	// Add icon
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 	GtkWidget* image = gtk_image_new_from_icon_name(m_icon, GTK_ICON_SIZE_DIALOG);
 	gtk_widget_show(image);
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
 	gtk_message_dialog_set_image(GTK_MESSAGE_DIALOG(dialog), image);
 G_GNUC_END_IGNORE_DEPRECATIONS
 
 	// Create accept button
-	gtk_dialog_add_button(dialog, m_mnemonic, GTK_RESPONSE_ACCEPT);
+	GtkWidget* button = gtk_dialog_add_button(dialog, m_mnemonic, GTK_RESPONSE_ACCEPT);
+	image = gtk_image_new_from_icon_name(m_icon, GTK_ICON_SIZE_BUTTON);
+	gtk_button_set_image(GTK_BUTTON(button), image);
 	gtk_dialog_set_default_response(dialog, GTK_RESPONSE_ACCEPT);
 
 	// Run dialog
@@ -262,7 +257,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
 	g_source_remove(timeout_id);
 	gtk_widget_destroy(m_timeout_details.dialog);
-	m_timeout_details.dialog = NULL;
+	m_timeout_details.dialog = nullptr;
 
 	return result == GTK_RESPONSE_ACCEPT;
 }
