@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2018 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,9 +26,14 @@ namespace WhiskerMenu
 class Command
 {
 public:
-	Command(const gchar* icon, const gchar* text, const gchar* command, const gchar* error_text, const gchar* confirm_question = NULL, const gchar* confirm_status = NULL);
+	Command(const gchar* icon, const gchar* text, const gchar* command, const gchar* error_text, const gchar* confirm_question = nullptr, const gchar* confirm_status = nullptr);
 	~Command();
 
+	Command(const Command&) = delete;
+	Command(Command&&) = delete;
+	Command& operator=(const Command&) = delete;
+	Command& operator=(Command&&) = delete;
+
 	GtkWidget* get_button();
 	GtkWidget* get_menuitem();
 
@@ -47,6 +52,11 @@ public:
 		return m_mnemonic;
 	}
 
+	const gchar* get_tooltip() const
+	{
+		return m_text;
+	}
+
 	void set(const gchar* command);
 
 	void set_shown(bool shown);
@@ -67,9 +77,16 @@ private:
 	gchar* m_text;
 	gchar* m_command;
 	gchar* m_error_text;
-	int m_status;
 	bool m_shown;
 
+	enum class CommandStatus
+	{
+		Unchecked,
+		Invalid,
+		Valid
+	}
+	m_status;
+
 	struct TimeoutDetails
 	{
 		GtkWidget* dialog;
