@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2014, 2015, 2016 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,7 +54,7 @@ static std::string normalize(const gchar* string)
 
 //-----------------------------------------------------------------------------
 
-static void replace_with_quoted_string(std::string& command, size_t& index, const gchar* unquoted)
+static void replace_with_quoted_string(std::string& command, std::string::size_type& index, const gchar* unquoted)
 {
 	if (!exo_str_is_empty(unquoted))
 	{
@@ -71,7 +71,7 @@ static void replace_with_quoted_string(std::string& command, size_t& index, cons
 
 //-----------------------------------------------------------------------------
 
-static void replace_with_quoted_string(std::string& command, size_t& index, const char* prefix, const gchar* unquoted)
+static void replace_with_quoted_string(std::string& command, std::string::size_type& index, const gchar* prefix, const gchar* unquoted)
 {
 	if (!exo_str_is_empty(unquoted))
 	{
@@ -91,7 +91,7 @@ static void replace_with_quoted_string(std::string& command, size_t& index, cons
 
 //-----------------------------------------------------------------------------
 
-static void replace_and_free_with_quoted_string(std::string& command, size_t& index, gchar* unquoted)
+static void replace_and_free_with_quoted_string(std::string& command, std::string::size_type& index, gchar* unquoted)
 {
 	replace_with_quoted_string(command, index, unquoted);
 	g_free(unquoted);
@@ -105,47 +105,17 @@ Launcher::Launcher(GarconMenuItem* item) :
 {
 	// Fetch icon
 	const gchar* icon = garcon_menu_item_get_icon_name(m_item);
-	if (G_LIKELY(icon))
-	{
-		if (!g_path_is_absolute(icon))
-		{
-			gchar* pos = g_strrstr(icon, ".");
-			if (!pos)
-			{
-				set_icon(icon);
-			}
-			else
-			{
-				gchar* suffix = g_utf8_casefold(pos, -1);
-				if ((strcmp(suffix, ".png") == 0)
-						|| (strcmp(suffix, ".xpm") == 0)
-						|| (strcmp(suffix, ".svg") == 0)
-						|| (strcmp(suffix, ".svgz") == 0))
-				{
-					set_icon(g_strndup(icon, pos - icon));
-				}
-				else
-				{
-					set_icon(icon);
-				}
-				g_free(suffix);
-			}
-		}
-		else
-		{
-			set_icon(icon);
-		}
-	}
+	set_icon(icon ? icon : "application-x-executable");
 
 	// Fetch text
 	const gchar* name = garcon_menu_item_get_name(m_item);
-	if (G_UNLIKELY(!name) || !g_utf8_validate(name, -1, NULL))
+	if (G_UNLIKELY(!name) || !g_utf8_validate(name, -1, nullptr))
 	{
 		name = "";
 	}
 
 	const gchar* generic_name = garcon_menu_item_get_generic_name(m_item);
-	if (G_UNLIKELY(!generic_name) || !g_utf8_validate(generic_name, -1, NULL))
+	if (G_UNLIKELY(!generic_name) || !g_utf8_validate(generic_name, -1, nullptr))
 	{
 		generic_name = "";
 	}
@@ -157,14 +127,14 @@ Launcher::Launcher(GarconMenuItem* item) :
 	m_display_name = name;
 
 	const gchar* details = garcon_menu_item_get_comment(m_item);
-	if (!details || !g_utf8_validate(details, -1, NULL))
+	if (!details || !g_utf8_validate(details, -1, nullptr))
 	{
 		details = generic_name;
 	}
 
 	// Create display text
 	const gchar* direction = (gtk_widget_get_default_direction() != GTK_TEXT_DIR_RTL) ? "\342\200\216" : "\342\200\217";
-	if (wm_settings->launcher_show_description)
+	if (wm_settings->launcher_show_description && !wm_settings->view_as_icons)
 	{
 		set_text(g_markup_printf_escaped("%s<b>%s</b>\n%s%s", direction, m_display_name, direction, details));
 	}
@@ -179,9 +149,22 @@ Launcher::Launcher(GarconMenuItem* item) :
 	m_search_generic_name = normalize(generic_name);
 	m_search_comment = normalize(details);
 
+	// Create search text for keywords
+#if GARCON_CHECK_VERSION(0,6,2)
+	GList* keywords = garcon_menu_item_get_keywords(m_item);
+	for (GList* i = keywords; i; i = i->next)
+	{
+		const gchar* keyword = static_cast<gchar*>(i->data);
+		if (!exo_str_is_empty(keyword) && g_utf8_validate(keyword, -1, nullptr))
+		{
+			m_search_keywords.push_back(normalize(keyword));
+		}
+	}
+#endif
+
 	// Create search text for command
 	const gchar* command = garcon_menu_item_get_command(m_item);
-	if (!exo_str_is_empty(command) && g_utf8_validate(command, -1, NULL))
+	if (!exo_str_is_empty(command) && g_utf8_validate(command, -1, nullptr))
 	{
 		m_search_command = normalize(command);
 	}
@@ -189,9 +172,9 @@ Launcher::Launcher(GarconMenuItem* item) :
 	// Fetch desktop actions
 #ifdef GARCON_TYPE_MENU_ITEM_ACTION
 	GList* actions = garcon_menu_item_get_actions(m_item);
-	for (GList* i = actions; i != NULL; i = i->next)
+	for (GList* i = actions; i; i = i->next)
 	{
-		GarconMenuItemAction* action = garcon_menu_item_get_action(m_item, reinterpret_cast<gchar*>(i->data));
+		GarconMenuItemAction* action = garcon_menu_item_get_action(m_item, static_cast<gchar*>(i->data));
 		if (action)
 		{
 			m_actions.push_back(new DesktopAction(action));
@@ -205,14 +188,61 @@ Launcher::Launcher(GarconMenuItem* item) :
 
 Launcher::~Launcher()
 {
-	for (std::vector<DesktopAction*>::size_type i = 0, end = m_actions.size(); i < end; ++i)
+	for (auto action : m_actions)
 	{
-		delete m_actions[i];
+		delete action;
 	}
 }
 
 //-----------------------------------------------------------------------------
 
+// Adapted from https://git.xfce.org/xfce/xfce4-appfinder/tree/src/appfinder-window.c#n945
+void Launcher::hide()
+{
+	// Look up the correct relative path
+	const gchar* relpath = nullptr;
+	gchar* uri = get_uri();
+	if (uri)
+	{
+		gchar** dirs = xfce_resource_lookup_all(XFCE_RESOURCE_DATA, "applications/");
+		for (size_t i = 0; dirs[i]; ++i)
+		{
+			if (g_str_has_prefix(uri + 7, dirs[i]))
+			{
+				relpath = uri + 7 + strlen(dirs[i]) - 13;
+				break;
+			}
+		}
+		g_strfreev(dirs);
+	}
+	if (!relpath)
+	{
+		g_free(uri);
+		return;
+	}
+
+	gchar* path = xfce_resource_save_location(XFCE_RESOURCE_DATA, relpath, false);
+	// I18N: the first %s will be replaced with desktop file path, the second with Hidden=true
+	gchar* message = g_strdup_printf(_("To unhide it you have to manually "
+			"remove the file \"%s\" or open the file and "
+			"remove the line \"%s\"."), path, "Hidden=true");
+	g_free(path);
+
+	if (xfce_dialog_confirm(nullptr, nullptr, _("Hide Application"), message,
+		_("Are you sure you want to hide \"%s\"?"), m_display_name))
+	{
+		XfceRc* rc = xfce_rc_config_open(XFCE_RESOURCE_DATA, relpath, false);
+		xfce_rc_set_group(rc, G_KEY_FILE_DESKTOP_GROUP);
+		xfce_rc_write_bool_entry(rc, G_KEY_FILE_DESKTOP_KEY_HIDDEN, true);
+		xfce_rc_close(rc);
+	}
+
+	g_free(message);
+	g_free(uri);
+}
+
+//-----------------------------------------------------------------------------
+
 void Launcher::run(GdkScreen* screen) const
 {
 	const gchar* string = garcon_menu_item_get_command(m_item);
@@ -228,8 +258,8 @@ void Launcher::run(GdkScreen* screen) const
 	}
 
 	// Expand the field codes
-	size_t length = command.length() - 1;
-	for (size_t i = 0; i < length; ++i)
+	std::string::size_type length = command.length() - 1;
+	for (std::string::size_type i = 0; i < length; ++i)
 	{
 		if (G_UNLIKELY(command[i] == '%'))
 		{
@@ -269,13 +299,13 @@ void Launcher::run(GdkScreen* screen) const
 
 	// Parse and spawn command
 	gchar** argv;
-	gboolean result = false;
-	GError* error = NULL;
-	if (g_shell_parse_argv(command.c_str(), NULL, &argv, &error))
+	bool result = false;
+	GError* error = nullptr;
+	if (g_shell_parse_argv(command.c_str(), nullptr, &argv, &error))
 	{
 		result = xfce_spawn_on_screen(screen,
 				garcon_menu_item_get_path(m_item),
-				argv, NULL, G_SPAWN_SEARCH_PATH,
+				argv, nullptr, G_SPAWN_SEARCH_PATH,
 				garcon_menu_item_supports_startup_notification(m_item),
 				gtk_get_current_event_time(),
 				garcon_menu_item_get_icon_name(m_item),
@@ -285,7 +315,7 @@ void Launcher::run(GdkScreen* screen) const
 
 	if (G_UNLIKELY(!result))
 	{
-		xfce_dialog_show_error(NULL, error, _("Failed to execute command \"%s\"."), string);
+		xfce_dialog_show_error(nullptr, error, _("Failed to execute command \"%s\"."), string);
 		g_error_free(error);
 	}
 }
@@ -302,8 +332,8 @@ void Launcher::run(GdkScreen* screen, DesktopAction* action) const
 	std::string command(string);
 
 	// Expand the field codes
-	size_t length = command.length() - 1;
-	for (size_t i = 0; i < length; ++i)
+	std::string::size_type length = command.length() - 1;
+	for (std::string::size_type i = 0; i < length; ++i)
 	{
 		if (G_UNLIKELY(command[i] == '%'))
 		{
@@ -343,13 +373,13 @@ void Launcher::run(GdkScreen* screen, DesktopAction* action) const
 
 	// Parse and spawn command
 	gchar** argv;
-	gboolean result = false;
-	GError* error = NULL;
-	if (g_shell_parse_argv(command.c_str(), NULL, &argv, &error))
+	bool result = false;
+	GError* error = nullptr;
+	if (g_shell_parse_argv(command.c_str(), nullptr, &argv, &error))
 	{
 		result = xfce_spawn_on_screen(screen,
 				garcon_menu_item_get_path(m_item),
-				argv, NULL, G_SPAWN_SEARCH_PATH,
+				argv, nullptr, G_SPAWN_SEARCH_PATH,
 				garcon_menu_item_supports_startup_notification(m_item),
 				gtk_get_current_event_time(),
 				action->get_icon(),
@@ -359,46 +389,56 @@ void Launcher::run(GdkScreen* screen, DesktopAction* action) const
 
 	if (G_UNLIKELY(!result))
 	{
-		xfce_dialog_show_error(NULL, error, _("Failed to execute command \"%s\"."), string);
+		xfce_dialog_show_error(nullptr, error, _("Failed to execute command \"%s\"."), string);
 		g_error_free(error);
 	}
 }
 
 //-----------------------------------------------------------------------------
 
-guint Launcher::search(const Query& query)
+unsigned int Launcher::search(const Query& query)
 {
 	// Prioritize matches in favorites and recent, then favories, and then recent
-	const guint flags = 3 - m_search_flags;
+	const unsigned int flags = 3 - m_search_flags;
 
 	// Sort matches in names first
-	guint match = query.match(m_search_name);
-	if (match != G_MAXUINT)
+	unsigned int match = query.match(m_search_name);
+	if (match != UINT_MAX)
 	{
 		return match | flags | 0x400;
 	}
 
 	match = query.match(m_search_generic_name);
-	if (match != G_MAXUINT)
+	if (match != UINT_MAX)
 	{
 		return match | flags | 0x800;
 	}
 
 	// Sort matches in comments next
 	match = query.match(m_search_comment);
-	if (match != G_MAXUINT)
+	if (match != UINT_MAX)
 	{
 		return match | flags | 0x1000;
 	}
 
-	// Sort matches in executables last
-	match = query.match(m_search_command);
-	if (match != G_MAXUINT)
+	// Sort matches in keywords next
+	for (const auto& keyword : m_search_keywords)
 	{
-		return match | flags | 0x2000;
+		match = query.match(keyword);
+		if (match != UINT_MAX)
+		{
+			return match | flags | 0x2000;
+		}
 	}
 
-	return G_MAXUINT;
+	// Sort matches in executables last
+	match = query.match(m_search_command);
+	if (match != UINT_MAX)
+	{
+		return match | flags | 0x4000;
+	}
+
+	return UINT_MAX;
 }
 
 //-----------------------------------------------------------------------------
