@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2014, 2015, 2016, 2017, 2018 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,12 @@
 #include "page.h"
 
 #include "favorites-page.h"
+#include "image-menu-item.h"
 #include "launcher.h"
-#include "launcher-view.h"
+#include "launcher-icon-view.h"
+#include "launcher-tree-view.h"
 #include "recent-page.h"
+#include "settings.h"
 #include "slot.h"
 #include "window.h"
 
@@ -34,17 +37,16 @@ using namespace WhiskerMenu;
 
 Page::Page(Window* window) :
 	m_window(window),
-	m_selected_path(NULL)
+	m_selected_launcher(nullptr),
+	m_drag_enabled(true),
+	m_launcher_dragged(false),
+	m_reorderable(false)
 {
 	// Create view
-	m_view = new LauncherView(window);
-	g_signal_connect_slot(m_view->get_widget(), "button-press-event", &Page::view_button_press_event, this);
-	g_signal_connect_slot(m_view->get_widget(), "popup-menu", &Page::view_popup_menu_event, this);
-	g_signal_connect_slot(m_view->get_widget(), "row-activated", &Page::item_activated, this);
-	g_signal_connect_swapped(m_view->get_widget(), "start-interactive-search", G_CALLBACK(gtk_widget_grab_focus), m_window->get_search_entry());
+	create_view();
 
 	// Add scrolling to view
-	m_widget = gtk_scrolled_window_new(NULL, NULL);
+	m_widget = gtk_scrolled_window_new(nullptr, nullptr);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_widget), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(m_widget), GTK_SHADOW_ETCHED_IN);
 	gtk_container_add(GTK_CONTAINER(m_widget), m_view->get_widget());
@@ -55,11 +57,6 @@ Page::Page(Window* window) :
 
 Page::~Page()
 {
-	if (m_selected_path)
-	{
-		gtk_tree_path_free(m_selected_path);
-	}
-
 	delete m_view;
 	gtk_widget_destroy(m_widget);
 	g_object_unref(m_widget);
@@ -71,31 +68,103 @@ void Page::reset_selection()
 {
 	m_view->collapse_all();
 
-	// Clear selection and scroll to top
+	// Set keyboard focus on first item
 	GtkTreeModel* model = m_view->get_model();
 	GtkTreeIter iter;
 	if (model && gtk_tree_model_get_iter_first(model, &iter))
 	{
 		GtkTreePath* path = gtk_tree_model_get_path(model, &iter);
-		get_view()->scroll_to_path(path);
-		get_view()->set_cursor(path);
+		m_view->set_cursor(path);
 		gtk_tree_path_free(path);
 	}
+
+	// Scroll to top
+	GtkAdjustment* adjustment = gtk_scrollable_get_vadjustment(GTK_SCROLLABLE(m_view->get_widget()));
+	gtk_adjustment_set_value(adjustment, gtk_adjustment_get_lower(adjustment));
+
+	// Clear selection
+	m_view->clear_selection();
+}
+
+//-----------------------------------------------------------------------------
+
+void Page::update_view()
+{
+	if (dynamic_cast<LauncherIconView*>(m_view) && wm_settings->view_as_icons)
+	{
+		return;
+	}
+
+	LauncherView* view = m_view;
+	create_view();
+	m_view->set_model(view->get_model());
+	delete view;
+
+	gtk_container_add(GTK_CONTAINER(m_widget), m_view->get_widget());
+	gtk_widget_show_all(m_widget);
+
+	view_created();
+}
+
+//-----------------------------------------------------------------------------
+
+void Page::set_reorderable(bool reorderable)
+{
+	m_reorderable = reorderable;
+	if (m_reorderable)
+	{
+		const GtkTargetEntry row_targets[] = {
+			{ g_strdup("GTK_TREE_MODEL_ROW"), GTK_TARGET_SAME_WIDGET, 0 },
+			{ g_strdup("text/uri-list"), GTK_TARGET_OTHER_APP, 1 }
+		};
+
+		m_view->set_drag_source(GDK_BUTTON1_MASK,
+				row_targets, 2,
+				GdkDragAction(GDK_ACTION_MOVE | GDK_ACTION_COPY));
+
+		m_view->set_drag_dest(row_targets, 1,
+				GDK_ACTION_MOVE);
+
+		g_free(row_targets[0].target);
+		g_free(row_targets[1].target);
+	}
+	else
+	{
+		const GtkTargetEntry row_targets[] = {
+			{ g_strdup("text/uri-list"), GTK_TARGET_OTHER_APP, 1 }
+		};
+
+		m_view->set_drag_source(GDK_BUTTON1_MASK,
+				row_targets, 1,
+				GDK_ACTION_COPY);
+
+		m_view->unset_drag_dest();
+
+		g_free(row_targets[0].target);
+	}
 }
 
 //-----------------------------------------------------------------------------
 
-Launcher* Page::get_selected_launcher() const
+void Page::create_view()
 {
-	Launcher* launcher = NULL;
-	if (m_selected_path)
+	if (wm_settings->view_as_icons)
 	{
-		GtkTreeModel* model = m_view->get_model();
-		GtkTreeIter iter;
-		gtk_tree_model_get_iter(model, &iter, m_selected_path);
-		gtk_tree_model_get(model, &iter, LauncherView::COLUMN_LAUNCHER, &launcher, -1);
+		m_view = new LauncherIconView();
+		g_signal_connect(m_view->get_widget(), "item-activated", G_CALLBACK(&Page::item_activated_slot), this);
 	}
-	return (launcher && (launcher->get_type() == Launcher::Type)) ? launcher : NULL;
+	else
+	{
+		m_view = new LauncherTreeView();
+		g_signal_connect(m_view->get_widget(), "row-activated", G_CALLBACK(&Page::row_activated_slot), this);
+		g_signal_connect_swapped(m_view->get_widget(), "start-interactive-search", G_CALLBACK(&gtk_widget_grab_focus), m_window->get_search_entry());
+	}
+	g_signal_connect_slot(m_view->get_widget(), "button-press-event", &Page::view_button_press_event, this);
+	g_signal_connect_slot(m_view->get_widget(), "button-release-event", &Page::view_button_release_event, this);
+	g_signal_connect_slot(m_view->get_widget(), "drag-data-get", &Page::view_drag_data_get, this);
+	g_signal_connect_slot(m_view->get_widget(), "drag-end", &Page::view_drag_end, this);
+	g_signal_connect_slot(m_view->get_widget(), "popup-menu", &Page::view_popup_menu_event, this);
+	set_reorderable(m_reorderable);
 }
 
 //-----------------------------------------------------------------------------
@@ -107,14 +176,14 @@ bool Page::remember_launcher(Launcher*)
 
 //-----------------------------------------------------------------------------
 
-void Page::item_activated(GtkTreeView* view, GtkTreePath* path, GtkTreeViewColumn*)
+void Page::launcher_activated(GtkTreePath* path)
 {
 	GtkTreeIter iter;
-	GtkTreeModel* model = gtk_tree_view_get_model(view);
+	GtkTreeModel* model = m_view->get_model();
 	gtk_tree_model_get_iter(model, &iter, path);
 
 	// Find element
-	Element* element = NULL;
+	Element* element = nullptr;
 	gtk_tree_model_get(model, &iter, LauncherView::COLUMN_LAUNCHER, &element, -1);
 	if (!element)
 	{
@@ -122,9 +191,8 @@ void Page::item_activated(GtkTreeView* view, GtkTreePath* path, GtkTreeViewColum
 	}
 
 	// Add to recent
-	if (element->get_type() == Launcher::Type)
+	if (Launcher* launcher = dynamic_cast<Launcher*>(element))
 	{
-		Launcher* launcher = static_cast<Launcher*>(element);
 		if (remember_launcher(launcher))
 		{
 			m_window->get_recent()->add(launcher);
@@ -135,60 +203,88 @@ void Page::item_activated(GtkTreeView* view, GtkTreePath* path, GtkTreeViewColum
 	m_window->hide();
 
 	// Execute app
-	element->run(gtk_widget_get_screen(GTK_WIDGET(view)));
+	element->run(gtk_widget_get_screen(m_widget));
 }
 
 //-----------------------------------------------------------------------------
 
-void Page::item_action_activated(GtkMenuItem* menuitem, DesktopAction* action)
+void Page::launcher_action_activated(GtkMenuItem* menuitem, DesktopAction* action)
 {
-	Launcher* launcher = get_selected_launcher();
-	if (!launcher)
-	{
-		return;
-	}
+	g_assert(m_selected_launcher);
 
 	// Add to recent
-	if (remember_launcher(launcher))
+	if (remember_launcher(m_selected_launcher))
 	{
-		m_window->get_recent()->add(launcher);
+		m_window->get_recent()->add(m_selected_launcher);
 	}
 
 	// Hide window
 	m_window->hide();
 
 	// Execute app
-	launcher->run(gtk_widget_get_screen(GTK_WIDGET(menuitem)), action);
+	m_selected_launcher->run(gtk_widget_get_screen(GTK_WIDGET(menuitem)), action);
 }
 
 //-----------------------------------------------------------------------------
 
-gboolean Page::view_button_press_event(GtkWidget* view, GdkEvent* event)
+gboolean Page::view_button_press_event(GtkWidget*, GdkEvent* event)
 {
-	GdkEventButton* event_button = reinterpret_cast<GdkEventButton*>(event);
-	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
-	GtkTreeIter iter;
-	if (gtk_tree_selection_get_selected(selection, NULL, &iter)
-			&& (event_button->type == GDK_BUTTON_PRESS)
-			&& (event_button->button == 3))
+	GdkEventButton* button_event = reinterpret_cast<GdkEventButton*>(event);
+
+	m_launcher_dragged = false;
+
+	GtkTreePath* path = m_view->get_path_at_pos(button_event->x, button_event->y);
+	if (!path)
 	{
-		create_context_menu(&iter, event);
+		return false;
+	}
+
+	if (button_event->button == 3)
+	{
+		create_context_menu(path, event);
 		return true;
 	}
+	else if (button_event->button != 1)
+	{
+		gtk_tree_path_free(path);
+		return false;
+	}
+
+	Element* element = nullptr;
+	GtkTreeModel* model = m_view->get_model();
+	GtkTreeIter iter;
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_path_free(path);
+	gtk_tree_model_get(model, &iter, LauncherView::COLUMN_LAUNCHER, &element, -1);
+	if (!(m_selected_launcher = dynamic_cast<Launcher*>(element)))
+	{
+		m_drag_enabled = false;
+		m_view->unset_drag_source();
+		m_view->unset_drag_dest();
+	}
+	else if (!m_drag_enabled)
+	{
+		m_drag_enabled = true;
+		set_reorderable(m_reorderable);
+	}
 
 	return false;
 }
 
 //-----------------------------------------------------------------------------
 
-gboolean Page::view_popup_menu_event(GtkWidget* view)
+gboolean Page::view_button_release_event(GtkWidget*, GdkEvent* event)
 {
-	GtkTreeSelection* selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
-	GtkTreeIter iter;
-	if (gtk_tree_selection_get_selected(selection, NULL, &iter))
+	GdkEventButton* button_event = reinterpret_cast<GdkEventButton*>(event);
+	if (button_event->button != 1)
 	{
-		create_context_menu(&iter, NULL);
-		return true;
+		return false;
+	}
+
+	if (m_launcher_dragged)
+	{
+		m_window->hide();
+		m_launcher_dragged = false;
 	}
 
 	return false;
@@ -196,39 +292,84 @@ gboolean Page::view_popup_menu_event(GtkWidget* view)
 
 //-----------------------------------------------------------------------------
 
-void Page::create_context_menu(GtkTreeIter* iter, GdkEvent* event)
+void Page::view_drag_data_get(GtkWidget*, GdkDragContext*, GtkSelectionData* data, guint info, guint)
 {
-	m_selected_path = gtk_tree_model_get_path(m_view->get_model(), iter);
-	Launcher* launcher = get_selected_launcher();
-	if (!launcher)
+	if ((info != 1) || !m_selected_launcher)
 	{
 		return;
 	}
 
+	gchar* uris[2] = { m_selected_launcher->get_uri(), nullptr };
+	if (uris[0])
+	{
+		gtk_selection_data_set_uris(data, uris);
+		g_free(uris[0]);
+	}
+
+	m_launcher_dragged = true;
+}
+
+//-----------------------------------------------------------------------------
+
+void Page::view_drag_end(GtkWidget*, GdkDragContext*)
+{
+	if (m_launcher_dragged)
+	{
+		m_window->hide();
+		m_launcher_dragged = false;
+	}
+}
+
+//-----------------------------------------------------------------------------
+
+gboolean Page::view_popup_menu_event(GtkWidget*)
+{
+	GtkTreePath* path = m_view->get_cursor();
+	if (!path)
+	{
+		return false;
+	}
+
+	create_context_menu(path, nullptr);
+
+	return true;
+}
+
+//-----------------------------------------------------------------------------
+
+void Page::create_context_menu(GtkTreePath* path, GdkEvent* event)
+{
+	// Get selected launcher
+	Element* element = nullptr;
+	GtkTreeModel* model = m_view->get_model();
+	GtkTreeIter iter;
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_model_get(model, &iter, LauncherView::COLUMN_LAUNCHER, &element, -1);
+	if (!(m_selected_launcher = dynamic_cast<Launcher*>(element)))
+	{
+		gtk_tree_path_free(path);
+		return;
+	}
+
 	// Create context menu
 	GtkWidget* menu = gtk_menu_new();
 	g_signal_connect_slot(menu, "selection-done", &Page::destroy_context_menu, this);
 
 	// Add menu items
-	GtkWidget* menuitem = gtk_menu_item_new_with_label(launcher->get_display_name());
+	GtkWidget* menuitem = gtk_menu_item_new_with_label(m_selected_launcher->get_display_name());
 	gtk_widget_set_sensitive(menuitem, false);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 	menuitem = gtk_separator_menu_item_new();
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-	const std::vector<DesktopAction*> actions = launcher->get_actions();
+	const auto actions = m_selected_launcher->get_actions();
 	if (!actions.empty())
 	{
-		for (std::vector<DesktopAction*>::size_type i = 0, end = actions.size(); i < end; ++i)
+		for (auto action : actions)
 		{
-			DesktopAction* action = actions[i];
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-			menuitem = gtk_image_menu_item_new_with_label(action->get_name());
-			GtkWidget* image = gtk_image_new_from_icon_name(action->get_icon(), GTK_ICON_SIZE_MENU);
-			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
-G_GNUC_END_IGNORE_DEPRECATIONS
-			g_signal_connect_slot(menuitem, "activate", &Page::item_action_activated, this, action);
+			menuitem = whiskermenu_image_menu_item_new(action->get_icon(), action->get_name());
+			g_signal_connect_slot(menuitem, "activate", &Page::launcher_action_activated, this, action);
 			gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 		}
 
@@ -236,60 +377,56 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	}
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-	if (!m_window->get_favorites()->contains(launcher))
+	if (!m_window->get_favorites()->contains(m_selected_launcher))
 	{
-		menuitem = gtk_image_menu_item_new_with_label(_("Add to Favorites"));
-		GtkWidget* image = gtk_image_new_from_icon_name("bookmark-new", GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
+		menuitem = whiskermenu_image_menu_item_new("bookmark-new", _("Add to Favorites"));
 		g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::add_selected_to_favorites, this);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	}
 	else
 	{
-		menuitem = gtk_image_menu_item_new_with_label(_("Remove From Favorites"));
-		GtkWidget* image = gtk_image_new_from_icon_name("list-remove", GTK_ICON_SIZE_MENU);
-		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
+		menuitem = whiskermenu_image_menu_item_new("list-remove", _("Remove From Favorites"));
 		g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::remove_selected_from_favorites, this);
 		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 	}
-G_GNUC_END_IGNORE_DEPRECATIONS
 
-	menuitem = gtk_menu_item_new_with_label(_("Add to Desktop"));
+	menuitem = whiskermenu_image_menu_item_new("list-add", _("Add to Desktop"));
 	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::add_selected_to_desktop, this);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-	menuitem = gtk_menu_item_new_with_label(_("Add to Panel"));
+	menuitem = whiskermenu_image_menu_item_new("list-add", _("Add to Panel"));
 	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::add_selected_to_panel, this);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
 	menuitem = gtk_separator_menu_item_new();
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
-	menuitem = gtk_menu_item_new_with_label(_("Edit Application..."));
+	menuitem = whiskermenu_image_menu_item_new("gtk-edit", _("Edit Application..."));
 	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::edit_selected, this);
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
 
+	menuitem = whiskermenu_image_menu_item_new("edit-delete", _("Hide Application"));
+	g_signal_connect_slot<GtkMenuItem*>(menuitem, "activate", &Page::hide_selected, this);
+	gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);
+
 	extend_context_menu(menu);
 
 	gtk_widget_show_all(menu);
 
 	// Show context menu
-	gtk_tree_view_set_hover_selection(GTK_TREE_VIEW(m_view->get_widget()), false);
-	gtk_menu_attach_to_widget(GTK_MENU(menu), m_view->get_widget(), NULL);
+	gtk_menu_attach_to_widget(GTK_MENU(menu), m_view->get_widget(), nullptr);
 	gtk_menu_popup_at_pointer(GTK_MENU(menu), event);
+
+	// Keep selection
+	m_view->select_path(path);
+	gtk_tree_path_free(path);
 }
 
 //-----------------------------------------------------------------------------
 
 void Page::destroy_context_menu(GtkMenuShell* menu)
 {
-	if (m_selected_path)
-	{
-		gtk_tree_path_free(m_selected_path);
-		m_selected_path = NULL;
-	}
-	gtk_tree_view_set_hover_selection(GTK_TREE_VIEW(m_view->get_widget()), true);
+	m_selected_launcher = nullptr;
 
 	gtk_widget_destroy(GTK_WIDGET(menu));
 
@@ -311,18 +448,17 @@ void Page::add_selected_to_desktop()
 	GFile* desktop_folder = g_file_new_for_path(desktop_path);
 
 	// Fetch launcher source
-	Launcher* launcher = get_selected_launcher();
-	g_assert(launcher != NULL);
-	GFile* source_file = launcher->get_file();
+	g_assert(m_selected_launcher);
+	GFile* source_file = m_selected_launcher->get_file();
 
 	// Fetch launcher destination
-	char* basename = g_file_get_basename(source_file);
+	gchar* basename = g_file_get_basename(source_file);
 	GFile* destination_file = g_file_get_child(desktop_folder, basename);
 	g_free(basename);
 
 	// Copy launcher to desktop folder
-	GError* error = NULL;
-	if (g_file_copy(source_file, destination_file, G_FILE_COPY_NONE, NULL, NULL, NULL, &error))
+	GError* error = nullptr;
+	if (g_file_copy(source_file, destination_file, G_FILE_COPY_NONE, nullptr, nullptr, nullptr, &error))
 	{
 		// Make launcher executable
 		gchar* path = g_file_get_path(destination_file);
@@ -331,7 +467,7 @@ void Page::add_selected_to_desktop()
 	}
 	else
 	{
-		xfce_dialog_show_error(NULL, error, _("Unable to add launcher to desktop."));
+		xfce_dialog_show_error(nullptr, error, _("Unable to add launcher to desktop."));
 		g_error_free(error);
 	}
 
@@ -345,21 +481,20 @@ void Page::add_selected_to_desktop()
 void Page::add_selected_to_panel()
 {
 	// Connect to Xfce panel through D-Bus
-	GError* error = NULL;
+	GError* error = nullptr;
 	GDBusProxy* proxy = g_dbus_proxy_new_for_bus_sync(G_BUS_TYPE_SESSION,
 			G_DBUS_PROXY_FLAGS_NONE,
-			NULL,
+			nullptr,
 			"org.xfce.Panel",
 			"/org/xfce/Panel",
 			"org.xfce.Panel",
-			NULL,
+			nullptr,
 			&error);
 	if (proxy)
 	{
 		// Fetch launcher desktop ID
-		Launcher* launcher = get_selected_launcher();
-		g_assert(launcher != NULL);
-		const gchar* parameters[] = { launcher->get_desktop_id(), NULL };
+		g_assert(m_selected_launcher);
+		const gchar* parameters[] = { m_selected_launcher->get_desktop_id(), nullptr };
 
 		// Tell panel to add item
 		if (!g_dbus_proxy_call_sync(proxy,
@@ -367,10 +502,10 @@ void Page::add_selected_to_panel()
 				g_variant_new("(s^as)", "launcher", parameters),
 				G_DBUS_CALL_FLAGS_NONE,
 				-1,
-				NULL,
+				nullptr,
 				&error))
 		{
-			xfce_dialog_show_error(NULL, error, _("Unable to add launcher to panel."));
+			xfce_dialog_show_error(nullptr, error, _("Unable to add launcher to panel."));
 			g_error_free(error);
 		}
 
@@ -379,7 +514,7 @@ void Page::add_selected_to_panel()
 	}
 	else
 	{
-		xfce_dialog_show_error(NULL, error, _("Unable to add launcher to panel."));
+		xfce_dialog_show_error(nullptr, error, _("Unable to add launcher to panel."));
 		g_error_free(error);
 	}
 }
@@ -388,27 +523,27 @@ void Page::add_selected_to_panel()
 
 void Page::add_selected_to_favorites()
 {
-	Launcher* launcher = get_selected_launcher();
-	g_assert(launcher != NULL);
-	m_window->get_favorites()->add(launcher);
+	g_assert(m_selected_launcher);
+	m_window->get_favorites()->add(m_selected_launcher);
 }
 
 //-----------------------------------------------------------------------------
 
 void Page::edit_selected()
 {
-	Launcher* launcher = get_selected_launcher();
-	g_assert(launcher != NULL);
+	g_assert(m_selected_launcher);
 
 	m_window->hide();
 
-	GError* error = NULL;
-	gchar* uri = launcher->get_uri();
-	gchar* command = g_strconcat("exo-desktop-item-edit ", uri, NULL);
+	GError* error = nullptr;
+	gchar* uri = m_selected_launcher->get_uri();
+	gchar* quoted_uri = g_shell_quote(uri);
+	gchar* command = g_strconcat("exo-desktop-item-edit ", quoted_uri, nullptr);
 	g_free(uri);
-	if (g_spawn_command_line_async(command, &error) == false)
+	g_free(quoted_uri);
+	if (!g_spawn_command_line_async(command, &error))
 	{
-		xfce_dialog_show_error(NULL, error, _("Unable to edit launcher."));
+		xfce_dialog_show_error(nullptr, error, _("Unable to edit launcher."));
 		g_error_free(error);
 	}
 	g_free(command);
@@ -416,11 +551,21 @@ void Page::edit_selected()
 
 //-----------------------------------------------------------------------------
 
-void Page::remove_selected_from_favorites()
+void Page::hide_selected()
 {
-	Launcher* launcher = get_selected_launcher();
-	g_assert(launcher != NULL);
-	m_window->get_favorites()->remove(launcher);
+	g_assert(m_selected_launcher);
+
+	m_window->hide();
+
+	m_selected_launcher->hide();
+}
+
+//-----------------------------------------------------------------------------
+
+void Page::remove_selected_from_favorites()
+{
+	g_assert(m_selected_launcher);
+	m_window->get_favorites()->remove(m_selected_launcher);
 }
 
 //-----------------------------------------------------------------------------
