@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2017, 2018 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,9 +31,14 @@ class Window;
 class Page
 {
 public:
-	explicit Page(WhiskerMenu::Window *window);
+	explicit Page(Window* window);
 	virtual ~Page();
 
+	Page(const Page&) = delete;
+	Page(Page&&) = delete;
+	Page& operator=(const Page&) = delete;
+	Page& operator=(Page&&) = delete;
+
 	GtkWidget* get_widget() const
 	{
 		return m_widget;
@@ -45,6 +50,7 @@ public:
 	}
 
 	void reset_selection();
+	void update_view();
 
 protected:
 	Window* get_window() const
@@ -52,11 +58,21 @@ protected:
 		return m_window;
 	}
 
+	virtual void view_created()
+	{
+	}
+
+	void set_reorderable(bool reorderable);
+
 private:
+	void create_view();
 	virtual bool remember_launcher(Launcher* launcher);
-	void item_activated(GtkTreeView* view, GtkTreePath* path, GtkTreeViewColumn*);
-	void item_action_activated(GtkMenuItem* menuitem, DesktopAction* action);
+	void launcher_activated(GtkTreePath* path);
+	void launcher_action_activated(GtkMenuItem* menuitem, DesktopAction* action);
 	gboolean view_button_press_event(GtkWidget* view, GdkEvent* event);
+	gboolean view_button_release_event(GtkWidget*, GdkEvent* event);
+	void view_drag_data_get(GtkWidget*, GdkDragContext*, GtkSelectionData* data, guint info, guint);
+	void view_drag_end(GtkWidget*, GdkDragContext*);
 	gboolean view_popup_menu_event(GtkWidget* view);
 	void on_unmap();
 	void destroy_context_menu(GtkMenuShell* menu);
@@ -64,16 +80,29 @@ private:
 	void add_selected_to_panel();
 	void add_selected_to_favorites();
 	void edit_selected();
+	void hide_selected();
 	void remove_selected_from_favorites();
-	Launcher* get_selected_launcher() const;
-	void create_context_menu(GtkTreeIter* iter, GdkEvent* event);
+	void create_context_menu(GtkTreePath* path, GdkEvent* event);
 	virtual void extend_context_menu(GtkWidget* menu);
 
+	static void item_activated_slot(GtkIconView*, GtkTreePath* path, gpointer user_data)
+	{
+		static_cast<Page*>(user_data)->launcher_activated(path);
+	}
+
+	static void row_activated_slot(GtkTreeView*, GtkTreePath* path, GtkTreeViewColumn*, gpointer user_data)
+	{
+		static_cast<Page*>(user_data)->launcher_activated(path);
+	}
+
 private:
 	Window* m_window;
 	GtkWidget* m_widget;
 	LauncherView* m_view;
-	GtkTreePath* m_selected_path;
+	Launcher* m_selected_launcher;
+	bool m_drag_enabled;
+	bool m_launcher_dragged;
+	bool m_reorderable;
 };
 
 }
