@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2014, 2015, 2016, 2017, 2018 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +36,11 @@ public:
 	explicit Plugin(XfcePanelPlugin* plugin);
 	~Plugin();
 
+	Plugin(const Plugin&) = delete;
+	Plugin(Plugin&&) = delete;
+	Plugin& operator=(const Plugin&) = delete;
+	Plugin& operator=(Plugin&&) = delete;
+
 	GtkWidget* get_button() const
 	{
 		return m_button;
