@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2015 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,17 +30,8 @@ class RunAction : public Element
 public:
 	RunAction();
 
-	enum
-	{
-		Type = 4
-	};
-	int get_type() const
-	{
-		return Type;
-	}
-
 	void run(GdkScreen* screen) const;
-	guint search(const Query& query);
+	unsigned int search(const Query& query);
 
 private:
 	std::string m_command_line;
