@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013, 2015 Graeme Gott <graeme@gottcode.org>
+ * Copyright (C) 2013-2020 Graeme Gott <graeme@gottcode.org>
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,13 +35,13 @@ public:
 	~SearchPage();
 
 	void set_filter(const gchar* filter);
-	void set_menu_items(GtkTreeModel* model);
+	void set_menu_items();
 	void unset_menu_items();
 
 private:
-	void activate_search();
-	void clear_search(GtkEntry* entry, GtkEntryIconPosition icon_pos, GdkEvent*);
-	gboolean cancel_search(GtkWidget* widget, GdkEvent* event);
+	void activate_search(GtkEntry* entry);
+	void cancel_search(GtkSearchEntry* entry);
+	void view_created();
 
 private:
 	Query m_query;
@@ -51,9 +51,9 @@ private:
 	class Match
 	{
 	public:
-		Match(Element* element = NULL) :
+		Match(Element* element = nullptr) :
 			m_element(element),
-			m_relevancy(G_MAXINT)
+			m_relevancy(UINT_MAX)
 		{
 		}
 
@@ -74,18 +74,18 @@ private:
 
 		void update(const Query& query)
 		{
-			g_assert(m_element != NULL);
+			g_assert(m_element);
 			m_relevancy = m_element->search(query);
 		}
 
 		static bool invalid(const Match& match)
 		{
-			return match.m_relevancy == G_MAXUINT;
+			return match.m_relevancy == UINT_MAX;
 		}
 
 	private:
 		Element* m_element;
-		guint m_relevancy;
+		unsigned int m_relevancy;
 	};
 	std::vector<Match> m_matches;
 };
